from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from .harvest_meta_data import HarvestMetaData
from .skos_concept import SkosConcept


@dataclass
class Event:
    id: Optional[str] = None
    uri: Optional[str] = None
    identifier: Optional[str] = None
    harvest: Optional[HarvestMetaData] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    dctType: Optional[List[SkosConcept]] = None
    relation: Optional[List[str]] = None
    associatedBroaderTypes: Optional[List[str]] = None
    mayInitiate: Optional[List[str]] = None
    subject: Optional[List[str]] = None
    distribution: Optional[List[str]] = None



from dataclasses import dataclass
from typing import (
    Any,
    List,
    Optional,
)

from .cpsvno_service import Service
from .organization import Organization


@dataclass
class PublicService(Service):
    hasCompetentAuthority: Optional[List[Organization]] = None

    def add_cpsvno_service_values(self: Any, values: Service) -> None:
        self.id = values.id
        self.uri = values.uri
        self.identifier = values.identifier
        self.title = values.title
        self.description = values.description
        self.harvest = values.harvest
        self.ownedBy = values.ownedBy
        self.contactPoint = values.contactPoint
        self.keyword = values.keyword
        self.sector = values.sector
        self.produces = values.produces
        self.spatial = values.spatial
        self.hasInput = values.hasInput
        self.processingTime = values.processingTime
        self.isDescribedAt = values.isDescribedAt
        self.hasParticipation = values.hasParticipation
        self.isGroupedBy = values.isGroupedBy
        self.isClassifiedBy = values.isClassifiedBy
        self.hasChannel = values.hasChannel
        self.follows = values.follows
        self.hasCost = values.hasCost
        self.requires = values.requires
        self.relation = values.relation
        self.hasLegalResource = values.hasLegalResource
        self.language = values.language
        self.holdsRequirement = values.holdsRequirement
        self.type = values.type
        self.admsStatus = values.admsStatus
        self.subject = values.subject
        self.homepage = values.homepage
        self.dctType = values.dctType
        self.thematicAreaUris = values.thematicAreaUris
        self.losThemes = values.losThemes
        self.euDataThemes = values.euDataThemes
        self.participatingAgents = values.participatingAgents



from typing import (
    List,
    Optional,
)

from fdk_rdf_parser.classes import (
    Agent,
    Channel,
    Evidence,
    Organization,
    Participation,
    PublicService,
    Service,
)
from .reference_data import PublicServiceReferenceData
from .utils import (
    extend_reference_data_code,
    extend_reference_data_code_list,
    remove_none_values,
)


def extend_cpsvno_service_with_reference_data(
    cpsvno_service: Service, ref_data: PublicServiceReferenceData
) -> Service:
    cpsvno_service.admsStatus = (
        extend_reference_data_code(cpsvno_service.admsStatus, ref_data.statuses)
        if cpsvno_service.admsStatus
        else None
    )
    cpsvno_service.hasInput = extend_cv_evidence(cpsvno_service.hasInput, ref_data)
    cpsvno_service.dctType = extend_reference_data_code_list(
        cpsvno_service.dctType, ref_data.types
    )
    cpsvno_service.hasChannel = extend_channels(cpsvno_service.hasChannel, ref_data)
    cpsvno_service.ownedBy = extend_organizations(cpsvno_service.ownedBy, ref_data)

    cpsvno_service.participatingAgents = extend_participations_of_agents(
        cpsvno_service.participatingAgents, ref_data
    )

    if isinstance(cpsvno_service, PublicService):
        cpsvno_service.hasCompetentAuthority = extend_organizations(
            cpsvno_service.hasCompetentAuthority, ref_data
        )

    return cpsvno_service


def extend_cv_evidence(
    evidences: Optional[List[Evidence]], ref_data: PublicServiceReferenceData
) -> Optional[List[Evidence]]:
    if evidences is None:
        return None

    for evidence in evidences:
        evidence.dctType = extend_reference_data_code_list(
            evidence.dctType, ref_data.evidence_type
        )
    return evidences


def extend_channels(
    channels: Optional[List[Channel]], ref_data: PublicServiceReferenceData
) -> Optional[List[Channel]]:
    if channels is None:
        return None

    for channel in channels:
        channel.channelType = (
            extend_reference_data_code(channel.channelType, ref_data.channel_type)
            if channel.channelType
            else None
        )
    return channels


def extend_organizations(
    organizations: Optional[List[Organization]], ref_data: PublicServiceReferenceData
) -> Optional[List[Organization]]:
    if organizations is None:
        return None

    for organization in organizations:
        organization.orgType = (
            extend_reference_data_code(
                organization.orgType, ref_data.organization_types
            )
            if organization.orgType
            else None
        )
    return organizations


def extend_participations_of_agents(
    participating_agents: Optional[List[Agent]], ref_data: PublicServiceReferenceData
) -> Optional[List[Agent]]:
    if participating_agents is None:
        return None

    for agent in participating_agents:
        agent.playsRole = extend_participations(agent.playsRole, ref_data)

    return participating_agents


def extend_participations(
    participations: Optional[List[Participation]], ref_data: PublicServiceReferenceData
) -> Optional[List[Participation]]:
    if participations is None:
        return None

    for participation in participations:
        extended_roles = []
        if participation.role:
            for role in participation.role:
                extended_roles.append(
                    extend_reference_data_code(role, ref_data.role_types)
                )
        filtered_extended_roles = remove_none_values(extended_roles)
        participation.role = (
            filtered_extended_roles if len(filtered_extended_roles) > 0 else None
        )

    return participations



from typing import (
    List,
    Optional,
)

from rdflib import (
    BNode,
    DC,
    DCTERMS,
    Graph,
    RDF,
    URIRef,
)

from fdk_rdf_parser.classes import MediaTypeOrExtent, MediaTypeOrExtentType
from fdk_rdf_parser.rdf_utils import (
    dcat_uri,
    dct_uri,
    euvoc_uri,
    object_value,
)


def extract_fdk_format(
    graph: Graph, subject: URIRef
) -> Optional[List[MediaTypeOrExtent]]:
    fdk_format: List[MediaTypeOrExtent] = list()
    dct_format = extract_media_type_list(graph, subject, dct_uri("format"))
    fdk_format.extend(dct_format if dct_format else [])
    dcat_media_type = extract_media_type_list(graph, subject, dcat_uri("mediaType"))
    fdk_format.extend(dcat_media_type if dcat_media_type else [])

    return fdk_format if len(fdk_format) > 0 else None


def media_type_name(graph: Graph, subject: URIRef) -> Optional[str]:
    dct_title = object_value(graph, subject, DCTERMS.title)
    if dct_title is not None:
        return dct_title
    else:
        return object_value(graph, subject, DC.identifier)


def media_type_code(graph: Graph, subject: URIRef) -> Optional[str]:
    dct_identifier = object_value(graph, subject, DCTERMS.identifier)
    if dct_identifier is not None:
        return dct_identifier
    else:
        return object_value(graph, subject, DC.identifier)


def media_type_type(graph: Graph, subject: URIRef) -> MediaTypeOrExtentType:
    rdf_type = graph.value(subject, RDF.type)
    if rdf_type == DCTERMS.MediaType:
        return MediaTypeOrExtentType.MEDIA_TYPE
    elif rdf_type == euvoc_uri("FileType"):
        return MediaTypeOrExtentType.FILE_TYPE
    else:
        return MediaTypeOrExtentType.UNKNOWN


def extract_media_type(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[MediaTypeOrExtent]:
    value = graph.value(subject, predicate)
    if not value:
        return None
    elif isinstance(value, BNode) or isinstance(value, URIRef):
        uri = value.toPython() if isinstance(value, URIRef) else None
        media_type = MediaTypeOrExtent(
            uri=uri,
            name=media_type_name(graph, value),
            code=media_type_code(graph, value),
            type=media_type_type(graph, value),
        )
        return media_type if media_type_is_valid(media_type) else None
    else:
        return MediaTypeOrExtent(code=value.toPython())


def extract_media_type_list(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[MediaTypeOrExtent]]:
    media_types = []
    for obj in graph.objects(subject, predicate):
        if isinstance(obj, BNode) or isinstance(obj, URIRef):
            uri = obj.toPython() if isinstance(obj, URIRef) else None
            media_type = MediaTypeOrExtent(
                uri=uri,
                name=media_type_name(graph, obj),
                code=media_type_code(graph, obj),
                type=media_type_type(graph, obj),
            )
            if media_type_is_valid(media_type):
                media_types.append(media_type)
        else:
            media_types.append(MediaTypeOrExtent(code=obj.toPython()))
    return media_types if len(media_types) > 0 else None


def media_type_is_valid(media_type: MediaTypeOrExtent) -> bool:
    if media_type == MediaTypeOrExtent():
        return False
    else:
        return True



from typing import (
    Any,
    List,
    Optional,
)

from rdflib import (
    Graph,
    Literal,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    RDF,
    SKOS,
)

from fdk_rdf_parser.classes import SkosConcept
from fdk_rdf_parser.rdf_utils import (
    object_value,
    resource_list,
    value_list,
    value_translations,
)


def extract_extra_type(graph: Graph, skos_concept: Any) -> Optional[str]:
    extra_type = None

    for type_uri_ref in resource_list(graph, skos_concept, RDF.type):
        if isinstance(type_uri_ref, URIRef) and type_uri_ref != SKOS.Concept:
            extra_type = type_uri_ref.toPython()

    return extra_type


def extract_skos_concept(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[SkosConcept]]:
    values = []
    for skos_concept in resource_list(graph, subject, predicate):
        skos_concept_uri = None
        if isinstance(skos_concept, URIRef) or isinstance(skos_concept, Literal):
            skos_concept_uri = skos_concept.toPython()

        source_value = object_value(graph, skos_concept, DCTERMS.source)

        values.append(
            SkosConcept(
                uri=source_value if source_value is not None else skos_concept_uri,
                prefLabel=value_translations(graph, skos_concept, SKOS.prefLabel),
                extraType=extract_extra_type(graph, skos_concept),
                broader=value_list(graph, skos_concept, SKOS.broader),
                narrower=value_list(graph, skos_concept, SKOS.narrower),
            )
        )

    return values if len(values) > 0 else None



from typing import (
    Dict,
    List,
    Optional,
)

from rdflib import (
    BNode,
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
    RDFS,
    SKOS,
)

from fdk_rdf_parser.classes import (
    Concept,
    Temporal,
)
from fdk_rdf_parser.classes.concept import (
    AssociativeRelation,
    Collection,
    Definition,
    GenericRelation,
    PartitiveRelation,
    Subject,
    TextAndURI,
)
from fdk_rdf_parser.rdf_utils import (
    date_value,
    dcat_uri,
    euvoc_uri,
    is_type,
    object_value,
    skosno_old_uri,
    skosno_uri,
    skosxl_uri,
    uneskos_uri,
    value_set,
    value_translations,
    xkos_uri_v_2,
)
from .contactpoint import extract_contact_points
from .harvest_meta_data import extract_meta_data
from .publisher import extract_publisher
from .temporal import extract_temporal


def parse_text_and_uri(graph: Graph, subject: URIRef) -> TextAndURI:
    return TextAndURI(
        text=value_translations(graph, subject, RDFS.label),
        uri=object_value(graph, subject, RDFS.seeAlso),
    )


def extract_range(graph: Graph, definition_ref: URIRef) -> Optional[TextAndURI]:
    range_list = []
    for range_ref in graph.objects(definition_ref, skosno_uri("omfang")):
        range_list.append(parse_text_and_uri(graph, range_ref))
    for range_ref in graph.objects(definition_ref, skosno_old_uri("omfang")):
        range_list.append(parse_text_and_uri(graph, range_ref))
    return range_list[0] if len(range_list) == 1 else None


def extract_sources(graph: Graph, definition_ref: URIRef) -> Optional[List[TextAndURI]]:
    sources = []
    for source_ref in graph.objects(definition_ref, DCTERMS.source):
        sources.append(parse_text_and_uri(graph, source_ref))
    return sources if len(sources) > 0 else None


def extract_subjects(graph: Graph, concept_ref: URIRef) -> Optional[List[Subject]]:
    subjects = []
    for subject_node in graph.objects(concept_ref, DCTERMS.subject):
        label: Optional[Dict[str, str]]
        if isinstance(subject_node, BNode) or isinstance(subject_node, URIRef):
            label = value_translations(graph, subject_node, SKOS.prefLabel)
        else:
            label = dict()
            if subject_node.language:
                label[subject_node.language] = subject_node.toPython()
            else:
                label["nb"] = subject_node.toPython()
        if label and len(label) > 0:
            subjects.append(Subject(label=label))
    return subjects if len(subjects) > 0 else None


def extract_status(graph: Graph, concept_ref: URIRef) -> Optional[Dict[str, str]]:
    status: Dict[str, str] = dict()
    for subject_node in graph.objects(concept_ref, euvoc_uri("status")):
        if isinstance(subject_node, BNode) or isinstance(subject_node, URIRef):
            return value_translations(graph, subject_node, SKOS.prefLabel)
        else:
            if subject_node.language:
                status[subject_node.language] = subject_node.toPython()
            else:
                status["nb"] = subject_node.toPython()
    return status if len(status) > 0 else None


def extract_target_group(graph: Graph, definition_ref: URIRef) -> Optional[str]:
    uri = object_value(graph, definition_ref, DCTERMS.audience)
    if not uri:
        return None
    if "allmennheten" in uri:
        return "allmennheten"
    elif "fagspesialist" in uri:
        return "fagspesialist"
    else:
        return None


def extract_source_relationship(graph: Graph, definition_ref: URIRef) -> Optional[str]:
    uri = object_value(graph, definition_ref, skosno_uri("forholdTilKilde"))
    uri = (
        uri
        if uri
        else object_value(graph, definition_ref, skosno_old_uri("forholdTilKilde"))
    )
    if not uri:
        return None
    if "sitatFraKilde" in uri:
        return "sitatFraKilde"
    elif "basertPåKilde" in uri:
        return "basertPåKilde"
    elif "egendefinert" in uri:
        return "egendefinert"
    else:
        return None


def parse_definition(graph: Graph, definition_ref: URIRef) -> Definition:
    return Definition(
        text=value_translations(graph, definition_ref, RDFS.label),
        remark=value_translations(graph, definition_ref, SKOS.scopeNote),
        targetGroup=extract_target_group(graph, definition_ref),
        sourceRelationship=extract_source_relationship(graph, definition_ref),
        range=extract_range(graph, definition_ref),
        sources=extract_sources(graph, definition_ref),
        lastUpdated=date_value(graph, definition_ref, DCTERMS.modified),
    )


def extract_definition(graph: Graph, concept_uri: URIRef) -> Optional[Definition]:
    definitions = []
    for definition_ref in graph.objects(concept_uri, skosno_uri("definisjon")):
        definitions.append(parse_definition(graph, definition_ref))
    for definition_ref in graph.objects(
        concept_uri, skosno_old_uri("betydningsbeskrivelse")
    ):
        definitions.append(parse_definition(graph, definition_ref))
    return definitions[0] if len(definitions) > 0 else None


def parse_associative_relation(
    graph: Graph, associative_relation_ref: URIRef
) -> AssociativeRelation:
    return AssociativeRelation(
        description=value_translations(
            graph, associative_relation_ref, DCTERMS.description
        ),
        related=object_value(graph, associative_relation_ref, SKOS.related),
    )


def extract_associative_relations(
    graph: Graph, concept_uri: URIRef
) -> Optional[List[AssociativeRelation]]:
    associative_relations = []
    for associative_relation_ref in graph.objects(
        concept_uri, skosno_uri("assosiativRelasjon")
    ):
        associative_relations.append(
            parse_associative_relation(graph, associative_relation_ref)
        )
    return associative_relations if len(associative_relations) > 0 else None


def parse_partitive_relation(
    graph: Graph, partitive_relation_ref: URIRef
) -> PartitiveRelation:
    return PartitiveRelation(
        description=value_translations(
            graph, partitive_relation_ref, DCTERMS.description
        ),
        hasPart=object_value(graph, partitive_relation_ref, DCTERMS.hasPart),
        isPartOf=object_value(graph, partitive_relation_ref, DCTERMS.isPartOf),
    )


def extract_partitive_relations(
    graph: Graph, concept_uri: URIRef
) -> Optional[List[PartitiveRelation]]:
    partitive_relations = []
    for partitive_relation_ref in graph.objects(
        concept_uri, skosno_uri("partitivRelasjon")
    ):
        partitive_relations.append(
            parse_partitive_relation(graph, partitive_relation_ref)
        )
    return partitive_relations if len(partitive_relations) > 0 else None


def parse_generic_relation(
    graph: Graph, generic_relation_ref: URIRef
) -> GenericRelation:
    return GenericRelation(
        divisioncriterion=value_translations(
            graph, generic_relation_ref, skosno_uri("inndelingskriterium")
        ),
        generalizes=object_value(
            graph, generic_relation_ref, xkos_uri_v_2("generalizes")
        ),
        specializes=object_value(
            graph, generic_relation_ref, xkos_uri_v_2("specializes")
        ),
    )


def extract_generic_relations(
    graph: Graph, concept_uri: URIRef
) -> Optional[List[GenericRelation]]:
    generic_relations = []
    for generic_relation_ref in graph.objects(
        concept_uri, skosno_uri("generiskRelasjon")
    ):
        generic_relations.append(parse_generic_relation(graph, generic_relation_ref))
    return generic_relations if len(generic_relations) > 0 else None


def parse_collection(graph: Graph, concept_record_uri: URIRef) -> Optional[Collection]:
    collection_record_uri = graph.value(concept_record_uri, DCTERMS.isPartOf)

    if collection_record_uri and is_type(
        dcat_uri("CatalogRecord"), graph, collection_record_uri
    ):
        collection_uri = graph.value(collection_record_uri, FOAF.primaryTopic)

        if collection_uri and is_type(SKOS.Collection, graph, collection_uri):
            return Collection(
                id=object_value(graph, collection_record_uri, DCTERMS.identifier),
                publisher=extract_publisher(graph, collection_uri),
                label=value_translations(graph, collection_uri, RDFS.label),
                uri=collection_uri.toPython(),
                description=value_translations(
                    graph, collection_uri, DCTERMS.description
                ),
            )
    return None


def create_application_language_dict(application: URIRef) -> Dict[str, str]:
    dict = {}
    if application.language:
        dict[application.language] = application.toPython()
    else:
        dict["nb"] = application.toPython()
    return dict


def parse_applications(
    graph: Graph, concept_uri: URIRef
) -> Optional[List[Dict[str, str]]]:
    applications = []
    for application in graph.objects(concept_uri, skosno_uri("bruksområde")):
        applications.append(create_application_language_dict(application))
    for application in graph.objects(concept_uri, skosno_old_uri("bruksområde")):
        applications.append(create_application_language_dict(application))
    return applications if len(applications) > 0 else None


def parse_label_set(
    graph: Graph, concept_uri: URIRef, predicate: URIRef
) -> Optional[List[Dict[str, str]]]:
    values = []
    for label in graph.objects(concept_uri, predicate):
        literal_form = value_translations(graph, label, skosxl_uri("literalForm"))
        if literal_form:
            values.append(literal_form)
    return values if len(values) > 0 else None


def parse_concept(graph: Graph, fdk_record_uri: URIRef, concept_uri: URIRef) -> Concept:
    concept_temporal_list = extract_temporal(graph, concept_uri)
    concept_temporal = concept_temporal_list[0] if concept_temporal_list else Temporal()
    contact_points = extract_contact_points(graph, concept_uri)

    pref_label_list = parse_label_set(graph, concept_uri, skosxl_uri("prefLabel"))

    return Concept(
        id=object_value(graph, fdk_record_uri, DCTERMS.identifier),
        uri=fdk_record_uri.toPython() if fdk_record_uri else None,
        identifier=concept_uri.toPython() if concept_uri else None,
        harvest=extract_meta_data(graph, fdk_record_uri),
        collection=parse_collection(graph, fdk_record_uri),
        publisher=extract_publisher(graph, concept_uri),
        subject=extract_subjects(graph, concept_uri),
        status=extract_status(graph, concept_uri),
        application=parse_applications(graph, concept_uri),
        example=value_translations(graph, concept_uri, SKOS.example),
        prefLabel=pref_label_list[0]
        if pref_label_list and len(pref_label_list) > 0
        else None,
        hiddenLabel=parse_label_set(graph, concept_uri, skosxl_uri("hiddenLabel")),
        altLabel=parse_label_set(graph, concept_uri, skosxl_uri("altLabel")),
        contactPoint=contact_points[0] if contact_points else None,
        definition=extract_definition(graph, concept_uri),
        seeAlso=value_set(graph, concept_uri, RDFS.seeAlso),
        isReplacedBy=value_set(graph, concept_uri, DCTERMS.isReplacedBy),
        replaces=value_set(graph, concept_uri, DCTERMS.replaces),
        validFromIncluding=concept_temporal.startDate,
        validToIncluding=concept_temporal.endDate,
        associativeRelation=extract_associative_relations(graph, concept_uri),
        partitiveRelation=extract_partitive_relations(graph, concept_uri),
        genericRelation=extract_generic_relations(graph, concept_uri),
        created=date_value(graph, concept_uri, DCTERMS.created),
        exactMatch=value_set(graph, concept_uri, SKOS.exactMatch),
        closeMatch=value_set(graph, concept_uri, SKOS.closeMatch),
        memberOf=value_set(graph, concept_uri, uneskos_uri("memberOf")),
    )



from dataclasses import dataclass
from typing import Optional

from .reference_data_code import ReferenceDataCode
from .skos_concept import SkosConcept


@dataclass
class Reference:
    referenceType: Optional[ReferenceDataCode] = None
    source: Optional[SkosConcept] = None



from dataclasses import dataclass
from enum import Enum
from typing import (
    Dict,
    List,
    Optional,
)

from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    dcat_uri,
)
from .reference_data_code import ReferenceDataCode


class EvidenceRdfType(str, Enum):
    UNKNOWN = "UNKNOWN"
    DATASET_TYPE = dcat_uri("Dataset")
    EVIDENCE_TYPE = cv_uri("Evidence")


@dataclass
class Evidence:
    rdfType: EvidenceRdfType = EvidenceRdfType.UNKNOWN
    uri: Optional[str] = None
    identifier: Optional[str] = None
    name: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    dctType: Optional[List[ReferenceDataCode]] = None
    language: Optional[List[ReferenceDataCode]] = None
    page: Optional[List[str]] = None



from functools import reduce
from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCAT,
    DCTERMS,
    SKOS,
)

from fdk_rdf_parser.classes import (
    BusinessEvent,
    Event,
    LifeEvent,
)
from fdk_rdf_parser.rdf_utils import (
    cpsvno_uri,
    cv_uri,
    is_type,
    object_value,
    resource_list,
    value_list,
    value_translations,
)
from .harvest_meta_data import extract_meta_data
from .skos_concept import extract_skos_concept


def extract_all_broader_skos_concepts(
    graph: Graph, subject: URIRef, values: List[str]
) -> List[str]:
    broader_list_uris = resource_list(graph, subject, SKOS.broader)

    values.append(subject.toPython())

    for skos_concept_uri in broader_list_uris:
        if skos_concept_uri.toPython() not in values:
            extract_all_broader_skos_concepts(graph, skos_concept_uri, values)

    return values


def extract_broader_types(graph: Graph, event_subject: URIRef) -> Optional[List[str]]:
    broader_type_instances_from_event = resource_list(
        graph, event_subject, DCTERMS.type
    )
    associated_skos_concepts_uris: List[str] = []
    if (
        broader_type_instances_from_event is not None
        and len(broader_type_instances_from_event) > 0
    ):
        reduce(
            lambda output, current: extract_all_broader_skos_concepts(
                graph, current, output
            ),
            broader_type_instances_from_event,
            associated_skos_concepts_uris,
        )

    return (
        associated_skos_concepts_uris
        if len(associated_skos_concepts_uris) > 0
        else None
    )


def parse_event(
    graph: Graph, catalog_record_uri: URIRef, subject: URIRef
) -> Optional[Event]:
    event = Event(
        id=object_value(graph, catalog_record_uri, DCTERMS.identifier),
        uri=subject.toPython(),
        identifier=object_value(graph, subject, DCTERMS.identifier),
        harvest=extract_meta_data(graph, catalog_record_uri),
        title=value_translations(graph, subject, DCTERMS.title),
        description=value_translations(graph, subject, DCTERMS.description),
        dctType=extract_skos_concept(graph, subject, DCTERMS.type),
        relation=value_list(graph, subject, DCTERMS.relation),
        associatedBroaderTypes=extract_broader_types(graph, subject),
        mayInitiate=value_list(graph, subject, cpsvno_uri("mayInitiate")),
        subject=value_list(graph, subject, DCTERMS.subject),
        distribution=value_list(graph, subject, DCAT.distribution),
    )

    if is_type(
        cv_uri("Event"),
        graph,
        subject,
    ):
        return event

    elif is_type(
        cv_uri("BusinessEvent"),
        graph,
        subject,
    ):
        business_event = BusinessEvent()
        business_event.add_event_values(event)
        return business_event

    else:
        life_event = LifeEvent()
        life_event.add_event_values(event)
        return life_event



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from fdk_rdf_parser.classes import ReferenceDataCode
from .reference_data_client import get_reference_data
from .utils import remove_scheme_and_trailing_slash


@dataclass
class DatasetReferenceData:
    provenancestatement: Optional[Dict[str, ReferenceDataCode]] = None
    rightsstatement: Optional[Dict[str, ReferenceDataCode]] = None
    frequency: Optional[Dict[str, ReferenceDataCode]] = None
    referencetypes: Optional[Dict[str, ReferenceDataCode]] = None


@dataclass
class PublicServiceReferenceData:
    week_days: Optional[Dict[str, ReferenceDataCode]] = None
    statuses: Optional[Dict[str, ReferenceDataCode]] = None
    types: Optional[Dict[str, ReferenceDataCode]] = None
    evidence_type: Optional[Dict[str, ReferenceDataCode]] = None
    channel_type: Optional[Dict[str, ReferenceDataCode]] = None
    organization_types: Optional[Dict[str, ReferenceDataCode]] = None
    role_types: Optional[Dict[str, ReferenceDataCode]] = None


def get_dataset_reference_data() -> DatasetReferenceData:
    return DatasetReferenceData(
        provenancestatement=get_and_map_provenance_statements(),
        rightsstatement=get_and_map_access_rights(),
        frequency=get_and_map_frequencies(),
        referencetypes=get_and_map_reference_types(),
    )


def get_public_service_reference_data() -> PublicServiceReferenceData:
    return PublicServiceReferenceData(
        week_days=get_and_map_week_days(),
        statuses=get_and_map_statuses(),
        evidence_type=get_and_map_evidence_types(),
        types=get_and_map_dct_types(),
        channel_type=get_and_map_channel_types(),
        organization_types=get_and_map_organization_types(),
        role_types=get_and_map_role_types(),
    )


def parse_reference_codes(
    codes: Optional[List[Dict]],
) -> Optional[Dict[str, ReferenceDataCode]]:
    if codes is not None:
        return {
            remove_scheme_and_trailing_slash(str(code.get("uri"))): ReferenceDataCode(
                uri=str(code.get("uri")) if code.get("uri") is not None else None,
                code=str(code.get("code")) if code.get("code") is not None else None,
                prefLabel=code.get("label") if code.get("label") is not None else None,
            )
            for code in codes
        }
    else:
        return None


def get_and_map_frequencies() -> Optional[Dict[str, ReferenceDataCode]]:
    frequencies = get_reference_data("/eu/frequencies").get("frequencies")
    return parse_reference_codes(frequencies)


def get_and_map_provenance_statements() -> Optional[Dict[str, ReferenceDataCode]]:
    provenance_statements = get_reference_data("/provenance-statements").get(
        "provenanceStatements"
    )
    return parse_reference_codes(provenance_statements)


def get_and_map_reference_types() -> Optional[Dict[str, ReferenceDataCode]]:
    reference_types = get_reference_data("/reference-types").get("referenceTypes")
    return parse_reference_codes(reference_types)


def get_and_map_access_rights() -> Optional[Dict[str, ReferenceDataCode]]:
    access_rights = get_reference_data("/eu/access-rights").get("accessRights")
    return parse_reference_codes(access_rights)


def get_and_map_week_days() -> Optional[Dict[str, ReferenceDataCode]]:
    week_days = get_reference_data("/schema/week-days").get("weekDays")
    return parse_reference_codes(week_days)


def get_and_map_statuses() -> Optional[Dict[str, ReferenceDataCode]]:
    statuses = get_reference_data("/adms/statuses").get("statuses")
    return parse_reference_codes(statuses)


def get_and_map_dct_types() -> Optional[Dict[str, ReferenceDataCode]]:
    dct_types = get_reference_data("/eu/main-activities").get("mainActivities")
    return parse_reference_codes(dct_types)


def get_and_map_channel_types() -> Optional[Dict[str, ReferenceDataCode]]:
    channels = get_reference_data("/digdir/service-channel-types").get(
        "serviceChannelTypes"
    )
    return parse_reference_codes(channels)


def get_and_map_organization_types() -> Optional[Dict[str, ReferenceDataCode]]:
    organization_types = get_reference_data("/adms/publisher-types").get(
        "publisherTypes"
    )
    return parse_reference_codes(organization_types)


def get_and_map_evidence_types() -> Optional[Dict[str, ReferenceDataCode]]:
    evidence_types = get_reference_data("/digdir/evidence-types").get("evidenceTypes")
    return parse_reference_codes(evidence_types)


def get_and_map_role_types() -> Optional[Dict[str, ReferenceDataCode]]:
    role_types = get_reference_data("/digdir/role-types").get("roleTypes")
    return parse_reference_codes(role_types)



from dataclasses import dataclass
from typing import (
    Dict,
    Optional,
)

from .reference_data_code import ReferenceDataCode


@dataclass
class Format:
    uri: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    format: Optional[str] = None
    language: Optional[ReferenceDataCode] = None
    seeAlso: Optional[str] = None



import os
from typing import (
    Dict,
    List,
    Optional,
    Sequence,
    TypeVar,
)

from fdk_rdf_parser.classes import (
    Reference,
    ReferenceDataCode,
)

base_url = os.getenv(
    "FDK_REFERENCE_DATA_BASE_URI",
    "https://www.staging.fellesdatakatalog.digdir.no",
)


def reference_data_url(endpoint: str) -> str:
    return f"{base_url}/reference-data{endpoint}"


def extend_reference_types(
    references: Optional[List[Reference]],
    reference_types: Optional[Dict[str, ReferenceDataCode]],
) -> Optional[List[Reference]]:
    if references is None or reference_types is None:
        return references
    else:
        extended_references = []
        for ref in references:
            ref.referenceType = (
                extend_reference_data_code(ref.referenceType, reference_types)
                if ref.referenceType
                else None
            )
            extended_references.append(ref)
        return extended_references


def extend_reference_data_code(
    ref_data_code: ReferenceDataCode,
    references: Optional[Dict[str, ReferenceDataCode]],
) -> ReferenceDataCode:
    ref_code = None
    if references:
        uri = ref_data_code.uri if ref_data_code else None
        ref_code = references.get(remove_scheme_and_trailing_slash(uri))

    return ref_code if ref_code else ref_data_code


def extend_reference_data_code_list(
    ref_data_codes: Optional[List[ReferenceDataCode]],
    references: Optional[Dict[str, ReferenceDataCode]],
) -> Optional[List[ReferenceDataCode]]:
    if ref_data_codes is None or references is None:
        return ref_data_codes

    extended_codes = []
    for code in ref_data_codes:
        ref_code = (
            references.get(remove_scheme_and_trailing_slash(code.uri))
            if code.uri
            else None
        )
        if ref_code:
            extended_codes.append(ref_code)
        else:
            extended_codes.append(code)
    return extended_codes


def remove_scheme_and_trailing_slash(uri: Optional[str]) -> str:
    if uri is None:
        return ""

    uri = uri.replace("http://", "").replace("https://", "")

    if uri.endswith("/"):
        return uri[:-1]
    else:
        return uri


T = TypeVar("T")


def remove_none_values(lst: Sequence[Optional[T]]) -> List[T]:
    return [item for item in lst if item is not None]



from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
)

from fdk_rdf_parser.classes import DataService
from fdk_rdf_parser.rdf_utils import (
    dcat_uri,
    object_value,
    value_list,
    value_set,
)
from .catalog import parse_catalog
from .dcat_resource import parse_dcat_resource
from .harvest_meta_data import extract_meta_data
from .media_type import extract_fdk_format
from .skos_concept import extract_skos_concept


def parse_data_service(
    data_services_graph: Graph, record_uri: URIRef, data_service_uri: URIRef
) -> DataService:
    data_service = DataService(
        id=object_value(data_services_graph, record_uri, DCTERMS.identifier),
        harvest=extract_meta_data(data_services_graph, record_uri),
        endpointURL=value_set(
            data_services_graph, data_service_uri, dcat_uri("endpointURL")
        ),
        endpointDescription=value_set(
            data_services_graph, data_service_uri, dcat_uri("endpointDescription")
        ),
        fdkFormat=extract_fdk_format(data_services_graph, data_service_uri),
        conformsTo=extract_skos_concept(
            data_services_graph, data_service_uri, DCTERMS.conformsTo
        ),
        servesDataset=value_set(
            data_services_graph, data_service_uri, dcat_uri("servesDataset")
        ),
        page=value_list(data_services_graph, data_service_uri, FOAF.page),
        catalog=parse_catalog(data_services_graph, record_uri),
    )

    data_service.add_values_from_dcat_resource(
        parse_dcat_resource(
            data_services_graph,
            data_service_uri,
        )
    )
    return data_service



from typing import Optional

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import Participation
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_value,
    value_translations,
)
from .reference_data_code import extract_reference_data_code_list


def parse_participation(graph: Graph, subject: URIRef) -> Optional[Participation]:
    resource_uri = subject.toPython() if isinstance(subject, URIRef) else None
    return Participation(
        uri=resource_uri,
        identifier=object_value(graph, subject, DCTERMS.identifier),
        description=value_translations(graph, subject, DCTERMS.description),
        role=extract_reference_data_code_list(graph, subject, cv_uri("role")),
        agent=object_value(graph, subject, cv_uri("hasParticipant")),
    )



from typing import (
    Dict,
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
)

from fdk_rdf_parser.classes import Agent
from fdk_rdf_parser.parse_functions.organization import parse_organization
from fdk_rdf_parser.parse_functions.participation import parse_participation
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_value,
    org_uri,
    resource_list,
    rov_uri,
    uri_or_identifier,
    value_translations,
)
from fdk_rdf_parser.rdf_utils.utils import is_type


def parse_agent_or_organization(graph: Graph, subject: URIRef) -> Optional[Agent]:
    if (
        is_type(org_uri("Organization"), graph, subject)
        or is_type(rov_uri("RegisteredOrganization"), graph, subject)
        or is_type(cv_uri("PublicOrganisation"), graph, subject)
    ):
        return parse_organization(graph, subject)
    elif is_type(FOAF.Agent, graph, subject) or is_type(DCTERMS.Agent, graph, subject):
        return Agent(
            uri=uri_or_identifier(graph, subject),
            identifier=object_value(graph, subject, DCTERMS.identifier),
            name=value_translations(graph, subject, DCTERMS.title),
        )
    else:
        return None


def extract_participating_agents(
    graph: Graph, service_uri: URIRef
) -> Optional[List[Agent]]:
    agents: Dict[str, Agent] = {}
    for participation in resource_list(graph, service_uri, cv_uri("hasParticipation")):
        agent_uri = object_value(graph, participation, cv_uri("hasParticipant"))
        if agent_uri and agent_uri not in agents:
            agent = parse_agent_or_organization(graph, URIRef(agent_uri))
            if agent:
                agents[agent_uri] = agent

        parsed_participation = parse_participation(graph, participation)
        agent = agents.get(agent_uri) if agent_uri else None
        if parsed_participation and agent:
            if agent.playsRole:
                agent.playsRole.append(parsed_participation)
            else:
                agent.playsRole = [parsed_participation]

    values = list(agents.values())
    return values if len(values) > 0 else None



from typing import (
    Dict,
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import SKOS

from fdk_rdf_parser.classes import (
    EuDataTheme,
    Eurovoc,
    LosNode,
)
from fdk_rdf_parser.rdf_utils import (
    fdk_internal_uri,
    resource_list,
    value_list,
    value_translations,
)

data_theme_base = "http://publications.europa.eu/resource/authority/data-theme"
eurovoc_base = "http://eurovoc.europa.eu"
los_base = "https://psi.norge.no/los"
los_theme_url_part = "/tema/"


def code_from_theme_uri(uri: str) -> str:
    return uri.split("/")[-1]


def split_theme_refs(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Dict[str, List[URIRef]]:
    los: List[URIRef] = list()
    data_themes: List[URIRef] = list()
    eurovocs: List[URIRef] = list()
    for resource in resource_list(graph, subject, predicate):
        theme_uri = resource.toPython() if isinstance(resource, URIRef) else None
        if los_base in theme_uri:
            los.append(resource)
        elif data_theme_base in theme_uri:
            data_themes.append(resource)
        elif eurovoc_base in theme_uri:
            eurovocs.append(resource)

    return {"los": los, "data-themes": data_themes, "eurovocs": eurovocs}


def map_los_themes(graph: Graph, theme_refs: List[URIRef]) -> Optional[List[LosNode]]:
    values = []
    for resource in theme_refs:
        los_uri = resource.toPython()

        values.append(
            LosNode(
                uri=los_uri,
                isTema=True if los_theme_url_part in los_uri else False,
                code=code_from_theme_uri(los_uri),
                name=value_translations(graph, resource, SKOS.prefLabel),
                losPaths=value_list(graph, resource, fdk_internal_uri("themePath")),
            )
        )

    return values if len(values) > 0 else None


def map_data_themes(
    graph: Graph, theme_refs: List[URIRef]
) -> Optional[List[EuDataTheme]]:
    values = []
    for resource in theme_refs:
        theme_uri = resource.toPython()
        values.append(
            EuDataTheme(
                uri=theme_uri,
                code=code_from_theme_uri(theme_uri),
                title=value_translations(graph, resource, SKOS.prefLabel),
            )
        )

    return values if len(values) > 0 else None


def map_eurovoc_themes(
    graph: Graph, theme_refs: List[URIRef]
) -> Optional[List[Eurovoc]]:
    values = []
    for resource in theme_refs:
        theme_uri = resource.toPython()
        values.append(
            Eurovoc(
                uri=theme_uri,
                code=code_from_theme_uri(theme_uri),
                label=value_translations(graph, resource, SKOS.prefLabel),
                eurovocPaths=value_list(graph, resource, fdk_internal_uri("themePath")),
            )
        )

    return values if len(values) > 0 else None



from typing import (
    List,
    Optional,
)

from rdflib import (
    BNode,
    Graph,
    URIRef,
)
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import ReferenceDataCode
from fdk_rdf_parser.rdf_utils import (
    object_value,
    resource_list,
    value_translations,
)


def extract_dct_spatial_list(
    graph: Graph, subject: URIRef
) -> Optional[List[ReferenceDataCode]]:
    spatial_list = []
    ref_list = resource_list(graph, subject, DCTERMS.spatial)
    for spatial_ref in ref_list:
        spatial = ReferenceDataCode(
            uri=spatial_ref.toPython() if not isinstance(spatial_ref, BNode) else None,
            code=object_value(graph, spatial_ref, DCTERMS.identifier),
            prefLabel=value_translations(graph, spatial_ref, DCTERMS.title),
        )
        if (
            spatial.uri is not None
            or spatial.code is not None
            or spatial.prefLabel is not None
        ):
            spatial_list.append(spatial)
    return spatial_list if len(spatial_list) > 0 else None



from dataclasses import dataclass
from enum import Enum
from typing import Optional


class MediaTypeOrExtentType(str, Enum):
    UNKNOWN = "UNKNOWN"
    MEDIA_TYPE = "MEDIA_TYPE"
    FILE_TYPE = "FILE_TYPE"


@dataclass
class MediaTypeOrExtent:
    uri: Optional[str] = None
    name: Optional[str] = None
    code: Optional[str] = None
    type: MediaTypeOrExtentType = MediaTypeOrExtentType.UNKNOWN



from .extend_cpsvno_service import extend_cpsvno_service_with_reference_data
from .extend_dataset import extend_dataset_with_reference_data
from .reference_data import (
    DatasetReferenceData,
    get_dataset_reference_data,
    get_public_service_reference_data,
    PublicServiceReferenceData,
)



from .agent import extract_participating_agents
from .catalog import parse_catalog
from .channel import extract_channels
from .concept import parse_concept
from .conforms_to import extract_conforms_to
from .contactpoint import extract_contact_points
from .cost import extract_costs
from .cpsvno_service import parse_cpsvno_service
from .dataservice import parse_data_service
from .dataset import (
    parse_dataset,
    parse_dataset_series,
)
from .dcat_resource import parse_dcat_resource
from .dct_standard import extract_dct_standard_list
from .distribution import extract_distributions
from .event import parse_event
from .evidence import extract_evidences
from .format import extract_formats
from .harvest_meta_data import extract_meta_data
from .info_model import parse_information_model
from .legal_resource import extract_legal_resources
from .output import extract_outputs
from .participation import parse_participation
from .publisher import extract_publisher
from .qualified_attribution import extract_qualified_attributions
from .quality_annotation import extract_quality_annotation
from .references import extract_references
from .requirement import extract_requirements
from .rule import extract_rules
from .skos_concept import extract_skos_concept
from .subject import extract_subjects
from .temporal import extract_temporal



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from .address import Address
from .reference_data_code import ReferenceDataCode


@dataclass
class Channel:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    channelType: Optional[ReferenceDataCode] = None
    description: Optional[Dict[str, str]] = None
    address: Optional[List[Address]] = None
    processingTime: Optional[str] = None
    hasInput: Optional[List[str]] = None
    email: Optional[List[str]] = None
    url: Optional[List[str]] = None
    telephone: Optional[List[str]] = None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
)

from fdk_rdf_parser.classes import Publisher
from fdk_rdf_parser.rdf_utils import (
    br_uri,
    object_value,
    rov_uri,
    value_translations,
)


def extract_publisher(graph: Graph, subject: URIRef) -> Optional[Publisher]:
    publisher = graph.value(subject, DCTERMS.publisher)
    return parse_publisher(graph, publisher) if publisher else None


def parse_publisher(graph: Graph, publisher: URIRef) -> Publisher:
    org_form = object_value(graph, publisher, rov_uri("orgType"))
    publisher = Publisher(
        uri=publisher.toPython() if isinstance(publisher, URIRef) else None,
        id=object_value(graph, publisher, DCTERMS.identifier),
        name=object_value(graph, publisher, rov_uri("legalName")),
        orgPath=object_value(graph, publisher, br_uri("orgPath")),
        prefLabel=value_translations(graph, publisher, FOAF.name),
        organisasjonsform=org_form.split("#")[-1] if org_form else None,
    )

    return set_publisher_name_from_pref_label_if_missing(publisher)


def extract_list_of_publishers(
    public_services_graph: Graph, public_service_uri: URIRef, predicate: URIRef
) -> Optional[List[Publisher]]:
    publisher_list = list(
        map(
            lambda publisher_uri: parse_publisher(public_services_graph, publisher_uri),
            public_services_graph.objects(public_service_uri, predicate),
        )
    )
    return publisher_list if len(publisher_list) > 0 else None


def set_publisher_name_from_pref_label_if_missing(publisher: Publisher) -> Publisher:
    if publisher.name or publisher.prefLabel is None:
        return publisher
    elif publisher.prefLabel.get("nb"):
        publisher.name = publisher.prefLabel.get("nb")
    elif publisher.prefLabel.get("no"):
        publisher.name = publisher.prefLabel.get("no")
    elif publisher.prefLabel.get("nn"):
        publisher.name = publisher.prefLabel.get("nn")
    elif publisher.prefLabel.get("en"):
        publisher.name = publisher.prefLabel.get("en")

    return publisher



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
    Set,
)

from .contactpoint import DCATContactPoint
from .publisher import Publisher
from .reference_data_code import ReferenceDataCode
from .theme import (
    EuDataTheme,
    Eurovoc,
    LosNode,
)


@dataclass
class PartialDcatResource:
    identifier: Optional[Set[str]] = None
    publisher: Optional[Publisher] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    descriptionFormatted: Optional[Dict[str, str]] = None
    uri: Optional[str] = None
    accessRights: Optional[ReferenceDataCode] = None
    themeUris: Optional[List[str]] = None
    theme: Optional[List[EuDataTheme]] = None
    losTheme: Optional[List[LosNode]] = None
    eurovocThemes: Optional[List[Eurovoc]] = None
    keyword: Optional[List[Dict[str, str]]] = None
    contactPoint: Optional[List[DCATContactPoint]] = None
    dctType: Optional[str] = None
    issued: Optional[str] = None
    modified: Optional[str] = None
    landingPage: Optional[Set[str]] = None
    language: Optional[List[ReferenceDataCode]] = None



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from .reference_data_code import ReferenceDataCode


@dataclass
class DCATContactPoint:
    uri: Optional[str] = None
    fullname: Optional[str] = None
    email: Optional[str] = None
    organizationName: Optional[Dict[str, str]] = None
    organizationUnit: Optional[Dict[str, str]] = None
    hasURL: Optional[str] = None
    hasTelephone: Optional[str] = None


@dataclass
class CVContactPoint:
    uri: Optional[str] = None
    contactType: Optional[Dict[str, str]] = None
    email: Optional[List[str]] = None
    telephone: Optional[List[str]] = None
    contactPage: Optional[List[str]] = None
    language: Optional[List[ReferenceDataCode]] = None



from dataclasses import dataclass
from typing import (
    Dict,
    Optional,
)


@dataclass
class Address:
    streetAddress: Optional[str] = None
    locality: Optional[str] = None
    postalCode: Optional[str] = None
    countryName: Optional[Dict[str, str]] = None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
    SKOS,
)

from fdk_rdf_parser.classes import Organization
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_value,
    org_uri,
    rov_uri,
)
from fdk_rdf_parser.rdf_utils.ns import br_uri
from fdk_rdf_parser.rdf_utils.utils import (
    is_type,
    value_list,
    value_translations,
)
from .dcat_resource import extract_reference_data_code


def parse_organization(graph: Graph, organization_uri: URIRef) -> Organization:
    organization = Organization(
        uri=organization_uri.toPython()
        if isinstance(organization_uri, URIRef)
        else None,
        identifier=object_value(graph, organization_uri, DCTERMS.identifier),
    )
    if is_type(org_uri("Organization"), graph, organization_uri) or is_type(
        cv_uri("PublicOrganisation"), graph, organization_uri
    ):
        organization.name = value_translations(graph, organization_uri, DCTERMS.title)
        organization.title = value_translations(graph, organization_uri, SKOS.prefLabel)
        organization.homepage = value_list(graph, organization_uri, FOAF.homepage)
        organization.spatial = value_list(graph, organization_uri, DCTERMS.spatial)
        organization.orgType = extract_reference_data_code(
            graph, organization_uri, DCTERMS.type
        )
    elif is_type(rov_uri("RegisteredOrganization"), graph, organization_uri):
        organization.name = value_translations(
            graph, organization_uri, rov_uri("legalName")
        )
        organization.orgPath = object_value(graph, organization_uri, br_uri("orgPath"))
        organization.title = value_translations(graph, organization_uri, FOAF.name)
        organization.homepage = value_list(graph, organization_uri, FOAF.homepage)
    return set_organization_name_from_pref_label_if_missing(organization)


def extract_organizations(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[Organization]]:
    organization_list = list(
        map(
            lambda organization_uri: parse_organization(graph, organization_uri),
            graph.objects(subject, predicate),
        )
    )
    return organization_list if len(organization_list) > 0 else None


def set_organization_name_from_pref_label_if_missing(
    organization: Organization,
) -> Organization:
    if (organization.name and len(organization.name) > 0) or organization.title is None:
        return organization
    organization.name = organization.title
    return organization



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import Channel
from fdk_rdf_parser.parse_functions.address import extract_address
from fdk_rdf_parser.rdf_utils import (
    object_value,
    resource_list,
)
from fdk_rdf_parser.rdf_utils.ns import (
    cpsv_uri,
    cv_uri,
    vcard_uri,
)
from fdk_rdf_parser.rdf_utils.utils import (
    duration_string_value,
    value_list,
    value_translations,
)
from .dcat_resource import extract_reference_data_code


def extract_channels(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[Channel]]:
    values = []
    for resource in resource_list(graph, subject, predicate):
        resource_uri = resource.toPython() if isinstance(resource, URIRef) else None
        values.append(
            Channel(
                uri=resource_uri,
                identifier=object_value(graph, resource, DCTERMS.identifier),
                channelType=extract_reference_data_code(graph, resource, DCTERMS.type),
                description=value_translations(graph, resource, DCTERMS.description),
                address=extract_address(graph, resource),
                processingTime=duration_string_value(
                    graph, resource, cv_uri("processingTime")
                ),
                hasInput=value_list(graph, resource, cpsv_uri("hasInput")),
                email=value_list(graph, resource, vcard_uri("hasEmail")),
                url=value_list(graph, resource, vcard_uri("hasURL")),
                telephone=value_list(graph, resource, vcard_uri("hasTelephone")),
            )
        )

    return values if len(values) > 0 else None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import Output
from fdk_rdf_parser.rdf_utils import (
    cpsv_uri,
    object_value,
    resource_list,
    value_translations,
)
from .reference_data_code import extract_reference_language_list
from .skos_concept import extract_skos_concept


def extract_outputs(graph: Graph, subject: URIRef) -> Optional[List[Output]]:
    values = []
    for resource in resource_list(graph, subject, cpsv_uri("produces")):
        resource_uri = resource.toPython() if isinstance(resource, URIRef) else None
        values.append(
            Output(
                uri=resource_uri,
                identifier=object_value(graph, resource, DCTERMS.identifier),
                name=value_translations(graph, resource, DCTERMS.title),
                description=value_translations(graph, resource, DCTERMS.description),
                language=extract_reference_language_list(
                    graph, resource, DCTERMS.language
                ),
                type=extract_skos_concept(graph, resource, DCTERMS.type),
            )
        )

    return values if len(values) > 0 else None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DCTERMS,
    FOAF,
)

from fdk_rdf_parser.classes.evidence import (
    Evidence,
    EvidenceRdfType,
)
from fdk_rdf_parser.rdf_utils import (
    cpsv_uri,
    cv_uri,
    dcat_uri,
    is_type,
    object_value,
    resource_list,
    uri_or_identifier,
    value_translations,
)
from fdk_rdf_parser.rdf_utils.utils import value_list
from .reference_data_code import (
    extract_reference_data_code_list,
    extract_reference_language_list,
)


def parse_evidence(graph: Graph, resource: URIRef) -> Evidence:
    if is_type(cv_uri("Evidence"), graph, resource):
        rdf_type = EvidenceRdfType.EVIDENCE_TYPE.value
    elif is_type(dcat_uri("Dataset"), graph, resource):
        rdf_type = EvidenceRdfType.DATASET_TYPE.value
    else:
        rdf_type = EvidenceRdfType.UNKNOWN.value

    return Evidence(
        rdfType=rdf_type,
        uri=uri_or_identifier(graph, resource),
        identifier=object_value(graph, resource, DCTERMS.identifier),
        name=value_translations(graph, resource, DCTERMS.title),
        description=value_translations(graph, resource, DCTERMS.description),
        dctType=extract_reference_data_code_list(graph, resource, DCTERMS.type),
        language=extract_reference_language_list(graph, resource, DCTERMS.language),
        page=value_list(graph, resource, FOAF.page),
    )


def extract_evidences(graph: Graph, subject: URIRef) -> Optional[List[Evidence]]:
    all_resources = resource_list(graph, subject, cpsv_uri("hasInput"))
    for channel in resource_list(graph, subject, cv_uri("hasChannel")):
        all_resources += resource_list(graph, channel, cpsv_uri("hasInput"))

    evidence_dict = dict()
    for resource in all_resources:
        evidence = parse_evidence(graph, resource)
        evidence_dict[evidence.uri] = evidence

    evidence_list = list(evidence_dict.values())
    return evidence_list if len(evidence_list) > 0 else None



from typing import (
    Dict,
    Optional,
)

from rdflib import Graph
from rdflib.namespace import (
    FOAF,
    RDF,
    SKOS,
)

from .classes import (
    Concept,
    DataService,
    Dataset,
    Event,
    InformationModel,
    Service,
)
from .parse_functions import (
    parse_concept,
    parse_cpsvno_service,
    parse_data_service,
    parse_dataset,
    parse_dataset_series,
    parse_event,
    parse_information_model,
)
from .rdf_utils import (
    cpsv_uri,
    cpsvno_uri,
    cv_uri,
    dcat_uri,
    is_type,
    model_dcat_ap_no_uri,
)
from .reference_data import (
    extend_cpsvno_service_with_reference_data,
    extend_dataset_with_reference_data,
    get_dataset_reference_data,
    get_public_service_reference_data,
)


def parse_data_services(
    data_service_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, DataService]:
    data_services: Dict[str, DataService] = {}

    data_services_graph = Graph().parse(data=data_service_rdf, format=rdf_format)

    for record_uri in data_services_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = data_services_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            dcat_uri("DataService"), data_services_graph, primary_topic_uri
        ):
            data_service = parse_data_service(
                data_services_graph, record_uri, primary_topic_uri
            )

            data_services[primary_topic_uri.toPython()] = data_service

    return data_services


def parse_datasets(rdf_data: str, rdf_format: str = "turtle") -> Dict[str, Dataset]:
    datasets_graph = Graph().parse(data=rdf_data, format=rdf_format)
    reference_data = get_dataset_reference_data()

    datasets: Dict[str, Dataset] = {}

    for record_uri in datasets_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = datasets_graph.value(record_uri, FOAF.primaryTopic)
        if primary_topic_uri is not None and (
            is_type(dcat_uri("Dataset"), datasets_graph, primary_topic_uri)
            or is_type(dcat_uri("DatasetSeries"), datasets_graph, primary_topic_uri)
        ):
            partial_dataset = parse_dataset(
                datasets_graph, record_uri, primary_topic_uri
            )

            dataset = Dataset()
            dataset.add_values_from_partial(values=partial_dataset)
            dataset = extend_dataset_with_reference_data(dataset, reference_data)

            if is_type(dcat_uri("DatasetSeries"), datasets_graph, primary_topic_uri):
                series = parse_dataset_series(datasets_graph, primary_topic_uri)
                series.add_values_from_dataset(values=dataset)
                datasets[primary_topic_uri.toPython()] = series
            else:
                datasets[primary_topic_uri.toPython()] = dataset

    return datasets


def parse_information_models(
    info_models_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, InformationModel]:
    info_models: Dict[str, InformationModel] = {}

    info_models_graph = Graph().parse(data=info_models_rdf, format=rdf_format)

    for record_uri in info_models_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = info_models_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            model_dcat_ap_no_uri("InformationModel"),
            info_models_graph,
            primary_topic_uri,
        ):
            info_model = parse_information_model(
                info_models_graph, record_uri, primary_topic_uri
            )

            info_models[primary_topic_uri.toPython()] = info_model

    return info_models


def parse_public_services(
    public_service_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, Service]:
    cpsvno_services: Dict[str, Service] = {}
    reference_data = get_public_service_reference_data()

    cpsvno_services_graph = Graph().parse(data=public_service_rdf, format=rdf_format)

    for catalog_record_uri in cpsvno_services_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = cpsvno_services_graph.value(
            catalog_record_uri, FOAF.primaryTopic
        )

        if primary_topic_uri and (
            is_type(
                cpsvno_uri("Service"),
                cpsvno_services_graph,
                primary_topic_uri,
            )
            or is_type(
                cpsv_uri("PublicService"),
                cpsvno_services_graph,
                primary_topic_uri,
            )
        ):
            cpsvno_service = parse_cpsvno_service(
                cpsvno_services_graph, catalog_record_uri, primary_topic_uri
            )

            cpsvno_service = extend_cpsvno_service_with_reference_data(
                cpsvno_service, reference_data
            )

            cpsvno_services[primary_topic_uri.toPython()] = cpsvno_service

    return cpsvno_services


def parse_events(
    event_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, Optional[Event]]:
    events: Dict[str, Optional[Event]] = {}
    graph = Graph().parse(data=event_rdf, format=rdf_format)

    for catalog_record_uri in graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = graph.value(catalog_record_uri, FOAF.primaryTopic)

        if primary_topic_uri and (
            is_type(
                cv_uri("Event"),
                graph,
                primary_topic_uri,
            )
            or is_type(
                cv_uri("BusinessEvent"),
                graph,
                primary_topic_uri,
            )
            or is_type(
                cv_uri("LifeEvent"),
                graph,
                primary_topic_uri,
            )
        ):
            event = parse_event(graph, catalog_record_uri, primary_topic_uri)

            events[primary_topic_uri.toPython()] = event

    return events


def parse_concepts(concepts_rdf: str, rdf_format: str = "turtle") -> Dict[str, Concept]:
    concepts: Dict[str, Concept] = {}
    concepts_graph = Graph().parse(data=concepts_rdf, format=rdf_format)

    for record_uri in concepts_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = concepts_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            SKOS.Concept,
            concepts_graph,
            primary_topic_uri,
        ):
            concept = parse_concept(concepts_graph, record_uri, primary_topic_uri)

            concepts[primary_topic_uri.toPython()] = concept

    return concepts



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from fdk_rdf_parser.classes.participation import Participation


@dataclass
class Agent:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    name: Optional[Dict[str, str]] = None
    playsRole: Optional[List[Participation]] = None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import Cost
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_number_value,
    object_value,
    resource_list,
    value_translations,
)
from .organization import extract_organizations


def extract_costs(graph: Graph, subject: URIRef) -> Optional[List[Cost]]:
    values = []
    for resource in resource_list(graph, subject, cv_uri("hasCost")):
        resource_uri = resource.toPython() if isinstance(resource, URIRef) else None
        values.append(
            Cost(
                uri=resource_uri,
                identifier=object_value(graph, resource, DCTERMS.identifier),
                description=value_translations(graph, resource, DCTERMS.description),
                currency=object_value(graph, resource, cv_uri("currency")),
                ifAccessedThrough=object_value(
                    graph, resource, cv_uri("ifAccessedThrough")
                ),
                isDefinedBy=extract_organizations(
                    graph, resource, cv_uri("isDefinedBy")
                ),
                value=str(object_number_value(graph, resource, cv_uri("value"))),
            )
        )

    return values if len(values) > 0 else None



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from fdk_rdf_parser.classes import Agent
from .reference_data_code import ReferenceDataCode


@dataclass
class Organization(Agent):
    title: Optional[Dict[str, str]] = None
    orgPath: Optional[str] = None
    orgType: Optional[ReferenceDataCode] = None
    spatial: Optional[List[str]] = None
    homepage: Optional[List[str]] = None



from typing import (
    Any,
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)

from fdk_rdf_parser.classes import (
    CVContactPoint,
    DCATContactPoint,
)
from fdk_rdf_parser.parse_functions.reference_data_code import (
    extract_reference_language_list,
)
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    dcat_uri,
    object_value,
    resource_list,
    value_list,
    value_translations,
    vcard_uri,
)


def extract_contact_points(
    graph: Graph, subject: URIRef
) -> Optional[List[DCATContactPoint]]:
    values = []
    for resource in resource_list(graph, subject, dcat_uri("contactPoint")):
        resource_uri = None
        if isinstance(resource, URIRef):
            resource_uri = resource.toPython()

        values.append(
            DCATContactPoint(
                uri=resource_uri,
                fullname=object_value(graph, resource, vcard_uri("fn")),
                email=extract_has_email(graph, resource),
                organizationName=value_translations(
                    graph, resource, vcard_uri("hasOrganizationName")
                ),
                organizationUnit=value_translations(
                    graph, resource, vcard_uri("organization-unit")
                ),
                hasURL=object_value(graph, resource, vcard_uri("hasURL")),
                hasTelephone=extract_has_telephone(graph, resource),
            )
        )

    return values if len(values) > 0 else None


def extract_has_telephone(graph: Graph, subject: Any) -> Optional[Any]:
    tel_ref = graph.value(subject, vcard_uri("hasTelephone"))
    if tel_ref:
        tel_value = object_value(graph, tel_ref, vcard_uri("hasValue"))
        if tel_value is None:
            tel_value = tel_ref.toPython()

        if "tel:" in tel_value:
            return tel_value[4:]
        else:
            return tel_value
    else:
        return None


def extract_has_email(graph: Graph, subject: Any) -> Optional[Any]:
    mail_ref = graph.value(subject, vcard_uri("hasEmail"))
    if mail_ref:
        email = object_value(graph, mail_ref, vcard_uri("hasValue"))
        if email is None:
            email = mail_ref.toPython()

        if "mailto:" in email:
            return email[7:]
        else:
            return email
    else:
        return None


def extract_cv_contact_point(
    graph: Graph, subject: URIRef
) -> Optional[List[CVContactPoint]]:
    values = []
    for resource in resource_list(graph, subject, cv_uri("contactPoint")):
        values.append(
            CVContactPoint(
                uri=resource.toPython() if isinstance(resource, URIRef) else None,
                contactType=value_translations(graph, resource, vcard_uri("category")),
                email=value_list(graph, resource, cv_uri("email")),
                telephone=value_list(graph, resource, cv_uri("telephone")),
                contactPage=value_list(graph, resource, cv_uri("contactPage")),
                language=extract_reference_language_list(
                    graph, resource, vcard_uri("language")
                ),
            )
        )

    return values if len(values) > 0 else None



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from fdk_rdf_parser.classes.theme import (
    EuDataTheme,
    Eurovoc,
    LosNode,
)
from .agent import Agent
from .channel import Channel
from .contactpoint import CVContactPoint
from .cost import Cost
from .evidence import Evidence
from .harvest_meta_data import HarvestMetaData
from .legal_resource import LegalResource
from .organization import Organization
from .output import Output
from .reference_data_code import ReferenceDataCode
from .requirement import Requirement
from .rule import Rule
from .skos_concept import SkosConcept


@dataclass
class Service:
    id: Optional[str] = None
    uri: Optional[str] = None
    identifier: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    harvest: Optional[HarvestMetaData] = None
    ownedBy: Optional[List[Organization]] = None
    contactPoint: Optional[List[CVContactPoint]] = None
    keyword: Optional[List[Dict[str, str]]] = None
    sector: Optional[List[SkosConcept]] = None
    produces: Optional[List[Output]] = None
    spatial: Optional[List[str]] = None
    hasInput: Optional[List[Evidence]] = None
    processingTime: Optional[str] = None
    isDescribedAt: Optional[List[SkosConcept]] = None
    hasParticipation: Optional[List[str]] = None
    isGroupedBy: Optional[List[str]] = None
    isClassifiedBy: Optional[List[SkosConcept]] = None
    hasChannel: Optional[List[Channel]] = None
    follows: Optional[List[Rule]] = None
    hasCost: Optional[List[Cost]] = None
    requires: Optional[List["Service"]] = None
    relation: Optional[List["Service"]] = None
    hasLegalResource: Optional[List[LegalResource]] = None
    language: Optional[List[ReferenceDataCode]] = None
    holdsRequirement: Optional[List[Requirement]] = None
    type: str = "publicservices"  # used by elasticsearch for indexing
    admsStatus: Optional[ReferenceDataCode] = None
    subject: Optional[List[SkosConcept]] = None
    homepage: Optional[List[str]] = None
    dctType: Optional[List[ReferenceDataCode]] = None
    thematicAreaUris: Optional[List[str]] = None
    losThemes: Optional[List[LosNode]] = None
    euDataThemes: Optional[List[EuDataTheme]] = None
    eurovocThemes: Optional[List[Eurovoc]] = None
    participatingAgents: Optional[List[Agent]] = None



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)

from .reference_data_code import ReferenceDataCode


@dataclass
class Participation:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    description: Optional[Dict[str, str]] = None
    role: Optional[List[ReferenceDataCode]] = None
    agent: Optional[str] = None



from rdflib import URIRef


def dcat_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/dcat#{sub_string}")


def adms_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/adms#{sub_string}")


def dcat_ap_no_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://difi.no/dcatno#{sub_string}")


def owl_time_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://www.w3.org/TR/owl-time/{sub_string}")


def schema_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://schema.org/{sub_string}")


def vcard_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/2006/vcard/ns#{sub_string}")


def uneskos_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://purl.org/umu/uneskos#{sub_string}")


def dct_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://purl.org/dc/terms/{sub_string}")


def dqv_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/dqv#{sub_string}")


def dqv_iso_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://iso.org/25012/2008/dataquality/{sub_string}")


def prov_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/prov#{sub_string}")


def dcat_api_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://dcat.no/dcatapi/{sub_string}")


def rov_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/regorg#{sub_string}")


def br_uri(sub_string: str) -> URIRef:
    return URIRef(
        f"https://raw.githubusercontent.com/Informasjonsforvaltning/organization-catalog/main/src/main/resources/ontology/organization-catalog.owl#{sub_string}"  # noqa: B950
    )


def fdk_uri(sub_string: str) -> URIRef:
    return URIRef(
        f"https://raw.githubusercontent.com/Informasjonsforvaltning/fdk-reasoning-service/main/src/main/resources/ontology/fdk.owl#{sub_string}"  # noqa: B950
    )


def fdk_internal_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://fellesdatakatalog.digdir.no/ontology/internal/{sub_string}")


def model_dcat_ap_no_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://data.norge.no/vocabulary/modelldcatno#{sub_string}")


def xkos_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://rdf-vocabulary.ddialliance.org/xkos/{sub_string}")


def xkos_uri_v_2(sub_string: str) -> URIRef:
    return URIRef(f"http://rdf-vocabulary.ddialliance.org/xkos#{sub_string}")


def cv_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://data.europa.eu/m8g/{sub_string}")


def cpsv_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://purl.org/vocab/cpsv#{sub_string}")


def cpsvno_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://data.norge.no/vocabulary/cpsvno#{sub_string}")


def skosno_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://data.norge.no/vocabulary/skosno#{sub_string}")


def skosno_old_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://difi.no/skosno#{sub_string}")


def skosxl_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/2008/05/skos-xl#{sub_string}")


def oa_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/oa#{sub_string}")


def prof_uri(sub_string: str) -> URIRef:
    return URIRef(f"https://www.w3.org/ns/dx/prof/{sub_string}")


def org_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://www.w3.org/ns/org#{sub_string}")


def euvoc_uri(sub_string: str) -> URIRef:
    return URIRef(f"http://publications.europa.eu/ontology/euvoc#{sub_string}")



from fdk_rdf_parser.classes import Dataset
from .reference_data import DatasetReferenceData
from .utils import (
    extend_reference_data_code,
    extend_reference_types,
)


def extend_dataset_with_reference_data(
    dataset: Dataset, ref_data: DatasetReferenceData
) -> Dataset:
    dataset.accessRights = (
        extend_reference_data_code(dataset.accessRights, ref_data.rightsstatement)
        if dataset.accessRights
        else None
    )
    dataset.provenance = (
        extend_reference_data_code(dataset.provenance, ref_data.provenancestatement)
        if dataset.provenance
        else None
    )
    dataset.accrualPeriodicity = (
        extend_reference_data_code(dataset.accrualPeriodicity, ref_data.frequency)
        if dataset.accrualPeriodicity
        else None
    )
    dataset.references = extend_reference_types(
        dataset.references, ref_data.referencetypes
    )

    return dataset



from typing import (
    List,
    Optional,
)

from rdflib import (
    BNode,
    Graph,
    URIRef,
)
from rdflib.namespace import (
    DC,
    SKOS,
)

from fdk_rdf_parser.classes import ReferenceDataCode
from fdk_rdf_parser.rdf_utils import (
    object_value,
    resource_list,
    value_list,
    value_translations,
)


def extract_reference_data_code(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[ReferenceDataCode]:
    uri = object_value(graph, subject, predicate)
    return ReferenceDataCode(uri=uri) if uri is not None else None


def extract_reference_data_code_list(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[ReferenceDataCode]]:
    ref_data_codes = []
    uri_list = value_list(graph, subject, predicate)
    if uri_list is None:
        return None
    else:
        for uri in uri_list:
            ref_data_codes.append(ReferenceDataCode(uri=uri))
        return ref_data_codes if len(ref_data_codes) > 0 else None


def extract_reference_language_list(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[ReferenceDataCode]]:
    lang_list = []
    ref_list = resource_list(graph, subject, predicate)
    for lang_ref in ref_list:
        lang = ReferenceDataCode(
            uri=lang_ref.toPython() if not isinstance(lang_ref, BNode) else None,
            code=object_value(graph, lang_ref, DC.identifier),
            prefLabel=value_translations(graph, lang_ref, SKOS.prefLabel),
        )
        if lang.uri is not None or lang.code is not None or lang.prefLabel is not None:
            lang_list.append(lang)
    return lang_list if len(lang_list) > 0 else None



from dataclasses import dataclass
from typing import (
    Dict,
    List,
    Optional,
)


@dataclass
class EuDataTheme:
    uri: Optional[str] = None
    code: Optional[str] = None
    title: Optional[Dict[str, str]] = None


@dataclass
class LosNode:
    isTema: Optional[bool] = None
    losPaths: Optional[List[str]] = None
    name: Optional[Dict[str, str]] = None
    uri: Optional[str] = None
    code: Optional[str] = None


@dataclass
class Eurovoc:
    uri: Optional[str] = None
    code: Optional[str] = None
    label: Optional[Dict[str, str]] = None
    eurovocPaths: Optional[List[str]] = None



from typing import (
    List,
    Optional,
)

from rdflib import (
    Graph,
    URIRef,
)

from fdk_rdf_parser.classes import Address
from fdk_rdf_parser.rdf_utils import (
    object_number_value,
    object_value,
    resource_list,
    value_translations,
)
from fdk_rdf_parser.rdf_utils.ns import vcard_uri


def extract_address(graph: Graph, subject: URIRef) -> Optional[List[Address]]:
    values = []
    for resource in resource_list(graph, subject, vcard_uri("hasAddress")):
        values.append(
            Address(
                streetAddress=object_value(
                    graph, resource, vcard_uri("street-address")
                ),
                locality=object_value(graph, resource, vcard_uri("locality")),
                postalCode=object_number_value(
                    graph, resource, vcard_uri("postal-code")
                ),
                countryName=value_translations(
                    graph, resource, vcard_uri("country-name")
                ),
            )
        )

    return values if len(values) > 0 else None
